{
open Header
open Ast
open Parser
}

rule token = parse
[' ' '\t' '\r']    { token lexbuf }
| [ '\n' ] 
{
    lineno := !lineno + 1;
    incr_linenum lexbuf;
    token lexbuf
}
| "(:" { comment lexbuf }
| '(' { LPAREN }
| ')' { RPAREN }
| '[' { LBRACK }
| ']' { RBRACK }
| '{' { LBRACE }
| '}' { RBRACE }
| ';' { SEMI }
| ':' { COLON }
| ',' { COMMA }
| '+' { PLUS }
| '-' { MINUS }
| '*' { TIMES }
| '/' { DIVIDE }
| '=' { ASSIGN }
| "==" { EQ }
| "!=" { NEQ }
| '<' { LT }
| "<=" { LEQ }
| '>' { GT }
| '.' { DOT }
| ">=" { GEQ }
| "Func" { FUNC }
| "Obj" { OBJ }
| "If" { IF }
| "End" { END }
| "Else" { ELSE }
| "Loop" { LOOP }
| "Break" { BREAK }
| "Return" { RETURN }
| "bool" { BOOL }
| "True" {TRUE}
| "False" {FALSE}
| "int" { INT }
| "WD"  { WD }
| "RGB" { RGB }
| "Not" { NOT }
| "FD" { FD }
| "BK" { BK }
| "LF" { LF }
| "RT" { RT }
| "RESET" { RESET }
| "CLS" { CLS }
| "PF" { PF }
| "PD" { PD }
| "PU" { PU }
| (['1'-'9']['0'-'9']* | '0') as lxm 
{
    try 
        INTV(int_of_string lxm) 
    with e -> 
        let errormsg = "Syntax Error: Integer '" ^ lxm ^ "' overflow at line: " ^ string_of_int !lineno in
        Printf.printf "%s\n"  errormsg;
        INTV(0)
}
| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm) }
| eof { EOF }
| _ as char
{
    let line = linecounter lexbuf in
    let col = columnstart lexbuf in
    raise ( IllegalCharacterError(Char.escaped char, line, col) );
}


and comment = parse
":)" { token lexbuf (* End of comment *) } 
| '\n' { lineno := !lineno + 1; incr_linenum lexbuf; token lexbuf (*line counter*) }
| _ { comment lexbuf (* Eat everything else *) }

